# %%
import supy as sp
import os
import opyplus as op
from pathlib import Path
import numpy as np
import f90nml
import pandas as pd

import datetime

import matplotlib.pyplot as plt
import matplotlib.font_manager as font_manager
import matplotlib.lines as mlines
import matplotlib.transforms as mtransforms
from matplotlib.colors import LogNorm
import matplotlib.gridspec as gridspec
import matplotlib.dates as mdates

import seaborn as sns

from sklearn import preprocessing

from STEBBS import LBM

# %%
def ext_conv_coeff(wind_speed: np.ndarray, dT=0.0) -> np.ndarray:
    """
    Calculates the external convection coefficient using Eq. 11 Cole & Sturrock (1977)

    Parameters
    ----------
    wind_speed : np.ndarray
        Wind speed [m s^-1]

    Returns
    -------
    np.ndarray
        hc :  external convection coefficient
    """

    # hc = 5.8 + 4.1 * wind_speed
    #DOE-2
    hn = 1.31 * (abs(dT) ** (1/3))
    a = 3.26
    b = 0.89
    Rf = 1.67 # Rough brick used from E+ Engineering Reference guide, Walton 1981
    hcglass = ((hn ** 2)+((a*(wind_speed ** b))** 2))** (0.5)
    hc = hn + Rf*(hcglass - hn)
    return hc

def create_building(nml_atype: tuple, vol: float, commercial=False) -> LBM:
    """
    Creates a STEBBS instances based on the buliding archtype parameters
    and building volumes.

    Parameters
    ----------
    nml_atype : tuple
        Tuple of group of f90nml NameLists and current building archetype
        reference
    vol : float
        Total volume of buildings in archetype
    commercial : bool
        True if creating a commercial BEM, False otherwise
    Returns
    -------
    LBM
        building : Instance of STEBBS BEM
    """

    # TODO: type of STEBBS instance
    # total volume of buildings of archetype <name> is found by the proportion of the number of such
    # dwellings in the neighbourhood TODO: change eq in paper to reflect this
    atype_nml = f90nml.read('./DomBuildingArchetypesBuildPeriods.nml')['BESTEST_Lightweight']
    build_width = float(vol / (atype_nml['Depth'] * atype_nml['Height']))
    
    building = LBM(nameListFile=atype_nml, w=build_width, d=atype_nml['Depth'], h=atype_nml['Height'])
    building.setHeatingCoolingSetpoint(Th=20.0+273.15, Tc=27.0+273.15)
    building.setMaxHeatingPower(1000000.0)

    if building.height == 0 or building.width == 0 or building.depth == 0:
        return None

    #Minimise Internal Mass volume for BESTEST
    building.setAirAndMassVolume(0.99)
    # nml gives ventilation in air changes per hour, but this need to be set to m3 s-1 and with building volume
    building.setVentilationRate(VR=building.airVolume * (building.ventilationRate/3600.)) # Value set to 0.018 m3/s in E+

    return building

####################### BESTEST EVALUTION - STEBBS vs. EPlus #######################

# ********run supy******** #

df_state_init, df_forcing=sp.load_SampleData()
df_output, df_state_final=sp.run_supy(df_forcing.loc['2012-01-01':'2013-01-01'],df_state_init)

 # *********************** #

# # ********generate EPW/TMY for EnergyPlus******** #
#EPlus Main Directory Path
eplus_dir_path = op.get_eplus_base_dir_path((9,3,0))
op.CONF.default_idd_version=9,3,0
#E+ Model Path
path_EP_base=Path('.').resolve() 
# #epw path
epw_base = Path('.').resolve()
epw_path = epw_base/'epw-ep.epw'

grid=df_state_init.index[0]
#*********** UNCOMMENT BELOW TO GENERATE EPW File ***********#
# df_suews=df_output.loc[epw=sp.util.gen_epwgrid,'SUEWS'].resample('1h').mean()
# df_epw, text_meta, path_(df_suews,path_epw=epw_path, lat=51.507, lon=0.128)
# epw_df = sp.util.read_epw(str(epw_path))
# print (epw_df.loc[(epw_df['Month'] == 1) & (epw_df['Day']  == 10) & (epw_df['Hour'] == 5)]) 
#*********** COMMENT ABOVE IF EPW FILE GENERATED ***********#

# ********************************************** #

# # ********Run EnergyPlus******** #

# #path to BETEST idf file
idf_path = path_EP_base/'Case600.idf'

# #run E+ simulation
s = op.simulate(str(idf_path), str(epw_path), "BESTEST_EP_9_3")

# #check run status
print(f"status: {s.get_status()}\n")
print(f"Eplus .err file:\n{s.get_out_err().get_content()}")

#Retrieve and display output
# retrieve hourly output (.eso file)
hourly_output = s.get_out_eso()

# ask for datetime index on year 2012
hourly_output.create_datetime_index(2012)

# get Pandas dataframe
df = hourly_output.get_data()

# monthly resample and display
# print(df.keys())
# print(df[[
#     "environment,Site Outdoor Air Drybulb Temperature",
#     "main zone,Zone Mean Air Temperature zone one"
# ]].resample("MS").mean())

# ***************************** #

# ********Run STEBBS******** #

# initialisation - Create BEM #
dom_atype_nml = f90nml.read('./DomBuildingArchetypesBuildPeriods.nml')

bem = create_building(nml_atype=(dom_atype_nml, 'BESTEST_600'), vol=(2.7*6.*8.)) # VOLUME HARDCODED

# Get BEM albedo and emisivity, set in SUEWS #
idx = pd.IndexSlice
last_ts = df_state_init.index.get_level_values(0)[-1]

# transfer `df_output` as forcing for STEBBS - extract forcing data #
T = \
    df_output.loc[
        idx[grid, :],
        idx['SUEWS', 'T2']]
T.name = 'Tair'
# tsurf
Tsurf = \
    df_output.loc[idx[grid, :],
                        idx['SUEWS', 'Tsurf']]
Tsurf.name = 'Tsurf'
# kdown, mean of dirs
Kwall = \
    df_output.loc[idx[grid, :],
                        ('SOLWEIG', ['Ksouth', 'Kwest', 'Knorth',
                                     'Keast'])].mean(axis=1)
Kwall.name = 'Kwall'
# kdown incident on roof
Kdown = \
    df_output.loc[idx[grid, :],
                        ('SOLWEIG', 'Kdown2d')]
Kdown.name = 'Kdown'
# windspeed
ws = \
    df_output.loc[idx[grid, :],
                        idx['SUEWS', 'U10']]

ws.name = 'ws'
envs = pd.concat([T, Tsurf, Kwall, Kdown, ws], axis=1)
envs = envs.droplevel(0).loc['2012-01-01':'2012-12-31']

#Resample Met Forcing to fit required timestep resolution
envsResample = envs.resample('1h').mean()

# # RUNNING WITH URBANISED FORCING ONLY 
run_stamps = envsResample.loc['2012-01-01':'2012-12-31'].index.values

qfres = pd.DataFrame(data= np.zeros(shape=(len(run_stamps), 5)), columns=['qheat_dom','qcool_dom','dom_tind', 
                                                                         'qfb_hw_dom', 'qfm_dom'],
                                    index=run_stamps)

# BOTH OCCUPANCY LEVELS AND APPLIANCE DATA ARE CONSTANT THROUGHOUT
bem.setOccupantData([0, 105, 0.43])  
bem.setApplianceData([200, 1, 1])

days_dict = {}


for t, tstep in enumerate(run_stamps):
#     df_output_forcing, df_state = \
#     sp.run_supy(df_forcing.loc['2013-10-03':'2015-09-30'],
#                 df_state_init)
    bem.setConvectionCoefficients(ho=[ext_conv_coeff(envs.loc[tstep, 'ws'],dT=(envs.loc[tstep, 'Tair']-envs.loc[tstep, 'Tsurf'])),ext_conv_coeff(envs.loc[tstep, 'ws'], dT=(envs.loc[tstep, 'Tair']-envs.loc[tstep, 'Tsurf']))])
    bem_qf = bem.timeStepCalculation_c(Ta=envs.loc[tstep, 'Tair'] + 273.15, Tg= 10 + 273.15,
                                       Tsurf=envs.loc[tstep, 'Tsurf'] + 273.15, rho_OutsideAir=1.225,
                                       Cp_OutsideAir=1005.0, Irr=envs.loc[tstep, 'Kdown'], 
                                       timestep=3600, resolution=1)
    
    bem_qf = np.array(bem_qf[0])
    qfm_dom = bem_qf[7] + bem_qf[8]  # this is just latent part for now
    qheat_dom = bem_qf[3] *3600
    qcool_dom = bem_qf[4] *3600
    qfb_hw_dom = bem_qf[6]
    dom_temp = bem.Ti

    qfres.loc[tstep, ['qheat_dom', 'qcool_dom', 'dom_tind', 'qfb_hw_dom', 'qfm_dom']] = \
                        [qheat_dom, qcool_dom, dom_temp, qfb_hw_dom, qfm_dom] 

# ************************** #
print("STEBBS HEATING (J): ", qfres['qheat_dom'].sum())
print("EPLUS HEATING (J): ", df['zone one,Zone Air System Sensible Heating Energy'].sum())
# ********BESTEST Comparison STEBBS, EPlus******** #
# 
# %%
#Sum


fig = plt.figure(figsize= (15,12))
grid_size = (4,2)
hosts_to_fmt = []

#Figure Title
subtitle_font = font_manager.FontProperties(family='sans-serif',style='normal',size=20)
plot_font = font_manager.FontProperties(family='sans-serif',style='normal',size=14)

# fig.text(x=0.8, y=0.95, s='BESTEST Comparison - STEBBS vs. EPlus: Case 600', \
#          fontproperties=subtitle_font, horizontalalignment='left', color='#524939')

#Top Figure Plots
gs1 = gridspec.GridSpec(3,2, left=0.08, top= 0.95, bottom=0.50, wspace=0)
large_top_ax = fig.add_subplot(gs1[0,:])#subplot2grid(grid_size, (0,0), colspan=2,rowspan=1)
large_middle_ax = fig.add_subplot(gs1[1,:], sharex=large_top_ax)#subplot2grid(grid_size, (1,0), colspan=2,rowspan=1, sharex=large_top_ax)
large_bottom_ax = fig.add_subplot(gs1[2,:], sharex=large_top_ax)#subplot2grid(grid_size, (2,0), colspan=2,rowspan=1, sharex=large_top_ax)

#Plot STEBBS and EnergyPlus Heating
large_top_ax.plot(qfres['qheat_dom'], color='#081616', marker='.', markersize=3, linestyle='None', label='STEBBS')
large_top_ax.plot(df['zone one,Zone Air System Sensible Heating Energy'], color='#C0DBDB', marker='.', markersize=3, linestyle='None', label='EnergyPlus9,3')
large_top_ax.text(.01,.85,'(a)', fontproperties=plot_font, horizontalalignment='left', transform=large_top_ax.transAxes)
large_top_ax.set_ylabel('Heating Load (J)', fontproperties=plot_font)
large_top_ax.legend() #redraw lengend to fix 'missing marker' issue
plt.setp(large_top_ax.get_xticklabels(), visible=False) #Switch x-axis labelling off

#Plot STEBBS and EnergyPlus Cooling
large_middle_ax.plot(qfres['qcool_dom'], color='#081616', marker='.', markersize=3, linestyle='None', label='STEBBS')
large_middle_ax.plot(df['zone one,Zone Air System Sensible Cooling Energy'], color='#C0DBDB', marker='.', markersize=3, linestyle='None', label='EnergyPlus9,3')
large_middle_ax.text(.01,.85,'(b)', fontproperties=plot_font, horizontalalignment='left', transform=large_middle_ax.transAxes)
large_middle_ax.set_ylabel('Cooling Load (J)', fontproperties=plot_font)
large_middle_ax.legend() #redraw lengend to fix 'missing marker' issue
plt.setp(large_middle_ax.get_xticklabels(), visible=False) #Switch x-axis labelling off

#Plot STEBBS and EnergyPlus Temperature
STEBBStempReadjusted = qfres['dom_tind']-273.15
large_bottom_ax.plot(STEBBStempReadjusted, color='#081616', marker='.', markersize=3, linestyle='None', label='STEBBS')
large_bottom_ax.plot(df['zone one,Zone Mean Air Temperature'], color='#C0DBDB', marker='.', markersize=3, linestyle='None', label='EnergyPlus9,3')
large_bottom_ax.text(.01,.85,'(c)', fontproperties=plot_font, horizontalalignment='left', transform=large_bottom_ax.transAxes)
large_bottom_ax.set_ylabel('Temperature ($^\circ$C)', fontproperties=plot_font)
large_bottom_ax.set_xlabel('Hours', fontproperties=plot_font)
large_bottom_ax.legend() #redraw legend to fix 'missing marker' issue

#x-axis formatting of date-time
large_bottom_ax.xaxis.set_major_locator(mdates.MonthLocator(bymonthday=1))
large_bottom_ax.xaxis.set_major_formatter(mdates.DateFormatter('%b'))
large_bottom_ax.set_xlim([datetime.date(2012, 1, 1), datetime.date(2012, 12, 31)])

########  DIFFERENCE PLOTTING  #########
df['HeatingDifference'] = df['zone one,Zone Air System Sensible Heating Energy'] - qfres['qheat_dom']
df['CoolingDifference'] = df['zone one,Zone Air System Sensible Cooling Energy'] - qfres['qcool_dom']
df['TempDifference'] = df['zone one,Zone Mean Air Temperature'] - STEBBStempReadjusted

gs2 = gridspec.GridSpec(1,2, left=0.08, top=0.45)
##########  1D DENSITY OR FREQUENCY PLOT  ##########
left_bottom_axis = fig.add_subplot(gs2[0,0])#subplot2grid(grid_size, (3,0))
left_bottom_axis.hist([df['HeatingDifference'].to_numpy(), df['CoolingDifference'].to_numpy()], bins=100, density=False, histtype='step', color=['#C0DBDB','#081616'], label=['Heating','Cooling'])
left_bottom_axis.text(.01,.95,'(d)', fontproperties=plot_font, horizontalalignment='left', transform=left_bottom_axis.transAxes)
left_bottom_axis.legend(title='Load Types')
left_bottom_axis.set_xlabel('Load differences EnergyPlus - STEBBS (J)', fontproperties=plot_font)
left_bottom_axis.set_ylabel('Frequency', fontproperties=plot_font)
##########  END 1D DENSITY OR FREQUENCY PLOT  ##########

##########  BOX PLOT  ##########
right_bottom_axis = fig.add_subplot(gs2[0,1])
WinterHeating = np.concatenate((df.loc['2012-01-01':'2012-03-31','HeatingDifference'].to_numpy(), df.loc['2012-10-01':'2012-12-31','HeatingDifference'].to_numpy()))
SummerHeating = df.loc['2012-05-01':'2012-08-31','HeatingDifference'].to_numpy()
WinterCooling = np.concatenate((df.loc['2012-01-01':'2012-03-31','CoolingDifference'].to_numpy(), df.loc['2012-10-01':'2012-12-31','HeatingDifference'].to_numpy()))
SummerCooling = df.loc['2012-05-01':'2012-08-31','CoolingDifference'].to_numpy()
bp = right_bottom_axis.boxplot([WinterHeating, SummerHeating, WinterCooling, SummerCooling], patch_artist=True, whis=[1,99])#, bootstrap=1000)
# Now add secondary axis for temperature boxplot
right_bottom_axis_2 = right_bottom_axis.twinx()
bp2 = right_bottom_axis_2.boxplot(df['TempDifference'], patch_artist=True, whis=[1,99], positions=[5])

for box in bp['boxes']:
   #change outline colour
   box.set(color='#808080', linewidth=2)
   #change fill colour
   box.set(facecolor = '#DCDCDC')
for box in bp2['boxes']:
   #change outline colour
   box.set(color='#808080', linewidth=2)
   #change fill colour
   box.set(facecolor = '#DCDCDC')

for whisker in bp['whiskers']:
   whisker.set(color='#808080', linewidth=2)
for whisker in bp2['whiskers']:
   whisker.set(color='#808080', linewidth=2)

for cap in bp['caps']:
   cap.set(color='#808080', linewidth=2)
for cap in bp2['caps']:
   cap.set(color='#808080', linewidth=2)

for median in bp['medians']:
   median.set(color='#2F4F4F', linewidth=2)
   x, y = median.get_xydata()[1]
   right_bottom_axis.text(x, y, '%.2f' %y, horizontalalignment='left', verticalalignment='center')
for median in bp2['medians']:
   median.set(color='#2F4F4F', linewidth=2)
   x, y = median.get_xydata()[1]
   right_bottom_axis_2.text(x, y, '%.2f' %y, horizontalalignment='left', verticalalignment='center')

for flier in bp['fliers']:
   flier.set(marker='+', color='#A9A9A9', alpha=0.5)
for flier in bp2['fliers']:
   flier.set(marker='+', color='#A9A9A9', alpha=0.5)

right_bottom_axis.set_xticklabels(['Winter Heat', 'Summer Heat', 'Winter Cool', 'Summer Cool', 'Temp'])

right_bottom_axis.get_xaxis().tick_bottom()
right_bottom_axis.get_yaxis().tick_left()

right_bottom_axis.text(.01,.95,'(e)', fontproperties=plot_font, horizontalalignment='left', transform=right_bottom_axis.transAxes)
right_bottom_axis.set_xlabel('Evaluation Metric', fontproperties=plot_font)
right_bottom_axis.set_ylabel('EPlus - STEBBS Difference (J)', fontproperties=plot_font)
right_bottom_axis_2.set_ylabel('$\Delta$ Indoor Temperature ($^\circ$C)', fontproperties=plot_font)

##########  END BOX PLOT  ##########

##########  END DIFFERENCE PLOTTING  ##########

#plt.tight_layout() #Use to get appropriate spacing between figures 
fig.savefig('./Evaluation.pdf', bbox_inches='tight')
plt.show()