//
//  STEBBS_C.c
//  
//  Simplified Thermal Energy Balance for Buildings Scheme (STEBBS)
//  Created by Stefan Thor Smith on 03/05/2018.
//

#include <stdio.h>
#include <Python.h>
#include "STEBBS_C.h"
#include <math.h>

static char module_docstring[] =
"This module provides an interface for calculating the thermal energy balance of the building using C.";

static char timeStep_docstring[] =
"Calculate the surface and air temperatures of all components in the building energy model. \n"
"All parameters to pass in are: \n"
"Ta - air temperature (K), \n"
"Tg - ground temperatur (K)e, \n"
"Tsurf - surrounding surface temperature (K),\n"
"rho_OutsideAir - outside air density,\n"
"Cp_OutsideAir - outside air specific heat capacity,\n"
"Irr - total downwelling irradiance,\n"
"winT - window transmissitance,\n"
"winA - window absorptance, \n"
"winR - window reflectance,\n"
"walT - wall transmittance,\n"
"walA - wall absorptance,\n"
"walR - wall reflectance,\n"
"Qf_sens[0] - current sensible Qf to air,\n"
"Qf_sens[1] - current sensible Qf to ground,\n"
"Qf_lat - current latent Qf,\n"
"Qheat - total heating system heat to indoor air,\n"
"Qcool - total cooling energy supplied to indoor (i.e. heat extracted from indoor),\n"
"ApplianceLoad - energy load from all appliances,\n"
"OccupantLoad[0] - occupant load (sensible),\n"
"OccupantLoad[1] - occupant load (latent),\n"
"height - building height,\n"
"width - building width,\n"
"depth - building depth,\n"
"WWR - window-to-wall ratio,\n"
"wallThickness - external wall thickness,\n"
"floorThickness - ground floor thickness,\n"
"groundDepth - ground depth for calculating heat conduction from floor,\n"
"windowThickness - window thickness, \n"
"wallRoofInternalConvectionCoefficient - internal wall effective convection coefficient,\n"
"internalMassConvectionCoefficient - internal mass effective convection coefficient,\n"
"floorInternalConvectionCoefficient - internal floor effective convection coefficient,\n"
"windowInternalConvectionCoefficient - internal window effective convection coefficient,\n"
"wallRoofExternalConvectionCoefficient - external wall effective convection coefficient,\n"
"windowExternalConvectionCoefficient - external window effective convection coefficient,\n"
"wallRoofEffectiveConductivity - wall and roof effective conductivity,\n"
"groundFloorEffectiveConductivity - ground floor effective conductivity,\n"
"windowEffectiveConductivity - window effective conductivity,\n"
"externalGroundConductivity - conductivity of ground under building,\n"
"wallRoofDensity - effective density of wall and roof,\n"
"groundFloorDensity - effective density of ground floor,\n"
"windowDensity - effective density of all windows,\n"
"internalMassDensity - effective density of all considered internal mass,\n"
"indoorAirDensity - density of air in building,\n"
"wallRoofSHC - specific heat capacity of external wall and roof combined,\n"
"groundFloorSHC - specific heat capacity of ground floor,\n"
"windowSHC - effective specific heat capacity of all windows,\n"
"internalMassSHC - effective specific heat capacity of all considered internal mass,"
"indoorAirSHC - specific heat capacity of air,"
"wallRoofExternalEmissivity - effective emissivity of external surface of building,"
"wallRoofInternalEmissivity - effective emissivity of internal surface of wall and roof, \n"
"internalMassEmissivity - effective emissivity of all considered internal mass,\n"
"windowExternalEmissivity - emissivity of outside window surfaces,\n"
"windowInternalEmissivity - emissivity of inside window surfaces,\n"
"windowTransmissivity - transmittance of windows,\n"
"windowAbsorbtivity - absorptance of windows,\n,"
"windowReflectivity - reflectance of  windows,\n"
"wallTransmisivity - transmittance of walls,\n"
"wallAbsorbtivity - absorptance of walls,\n"
"wallReflectivity - reflectance of walls,\n"
"outdoorWallBVF - view factor of surrounding buildings for the wall,\n"
"outdoorWallGVF - view factor of the ground for the wall of the building,\n"
"outdoorWallSVF - view factor of the sky for the wall of the building,\n"
"Occupants - number of occupants,\n"
"MetRate - metabolic rate of occupants,\n"
"LSR - latent to sensible heat ratio related to occupant metabolic rate,\n"
"ApplianceRating - power rating of appliances,\n"
"applianceUsageFactor - usage factor of appliances,\n"
"maxHeatingPower - sizing of the heating system in the building,\n"
"heatingEfficiency, - heating system efficiency,\n"
"maxCoolingPower - sizing of the cooling system,\n"
"coolingCOP - coefficient of performance of the cooling system,\n"
"airVolume - volume of air in the building,\n"
"ventilationRate - ventilation rate for the whole building,\n"
"wallArea - total wall area of building,\n "
"wallVolume - total external wall volume of building,\n"
"floorArea - ground floor area,\n"
"floorVolume - total ground floor volume,\n"
"windowArea - total window area,\n"
"windowVolume - total window volume,\n"
"internalMassVolume - total volume of internal mass,\n"
"internalMassArea - total surface area of internal mass,\n"
"Ti - indoor air temperature (K),\n"
"Tm - internal mass temperature (K),\n"
"Twi - wall internal surface layer temperature (K),\n"
"Two - wall outside surface layer temperature (K),\n"
"Twii - window internal surface layer temperature (K),\n"
"Twio - window outside surface layer temperature (K),\n"
"Tfi - floor indoor surface layer temperature (K),\n"
"Tfo - floor outdoor surface layer temperature (K),\n"
"Ts[0] - heating set point temperature (K),\n"
"Ts[1] - cooling set point temperature (K),\n"
"totalAppliances - total number of appliances in use,\n"
"timestep - length of single timestep calculation (s),\n"
"resolution - resolution of model calculations (s)\n"
"qhwtTot - total heating of hot water over course of simulation (J)\n"
"Twt - temperature of water in hot water tank (K) \n"
"Twt_wallIn - temperature of internal wall of hot water tank (K) \n"
"Twt_wallExt - temperature of external wall in hot water tank (K) \n"
"hwtThickness - thickness of hot water tank wall (m)\n"
"Twt_in - temperature of water fed into hot water tank (i.e. mains water temperature) (K) \n"
"Vwt - volume of water in hot water tank (m^3)\n"
"Awt - surface area of hot water tank (m^2) \n"
"wVwt - total volume of wall of hot water tank (m^3)\n"
"wtTs - set point temperature (K) for water held in hot water tank (m^3)\n"
"Tdhw - temperature of Domestic Hot Water held in use in Building (K) \n"
"Tdhw_wallIn - temperature of internal wall of vessels holding DHW (K) \n"
"Tdhw_wallExt - temperature of external wall of vessels holding DHW (K) \n"
"dhwThickness - thickness of vessels holding DHW in use in Building (m)\n"
"Vdhw - volume of DHW in use in building (m^3)\n"
"Adhw - area of DHW in use in building (m^2)\n"
"wVdhw\n"
"VwtFlowRate\n"
"VdhwFlowRate\n"
"DHW_SHC\n"
"HWT_SHC\n"
"VHW_SHC\n"
"DHW_Density\n"
"HWT_Density\n"
"VHW_Density\n"
"WTBVF\n"
"WTMVF\n"
"hotWaterTankWallConductivity\n"
"hotWaterTankWallConvectionCoefficient_Internal\n"
"hotWaterTankWallConvectionCoefficient_External\n"
"hotWaterTankWallEmissivity_External\n"
"domesticHotWaterConductivity\n"
"domesticHotWaterVesselsConvectionCoefficient_Internal\n"
"domesticHotWaterVesselsConvectionCoefficient_External\n"
"domesticHotWaterVesselsEmissivity_External\n"
"maxHeatingPower_DHW\n"
"DHW_HeatingEfficiency\n"
"minVdhw";

static PyObject *timeStepCalculation(PyObject *self, PyObject * args);

static PyMethodDef Methods[] = {
    
    {"tstep", timeStepCalculation, METH_VARARGS, timeStep_docstring},
    
    {NULL, NULL} /*Sentinel*/
    
};

//Used if PY_MAJOR_VERSION >=3
#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef moduledef = {
    PyModuleDef_HEAD_INIT,
    "STEBBS_C",
    timeStep_docstring, //docstring for the module
    -1, //Note this is set to this as sub-interpreters not supported at this stage
    Methods,
    NULL,
    NULL, //Assume traversal function is not needed
    NULL, //Assume lear function is not needed
    NULL // Assume a deallocation function is not needed
};

#define INITERROR return NULL
PyMODINIT_FUNC
PyInit_STEBBS_C()

#else //Used if version 2.X
#define INITERROR return
void initSTEBBS_C()
#endif
{
#if PY_MAJOR_VERSION >= 3
    (void) PyModule_Create(&moduledef);
#else
    (void) Py_InitModule("STEBBS_C", Methods);
#endif
}

/*******************************************************************/
//Functions for calculating heat transfer in Building Energy lumped component model
/*******************************************************************/

/*******************************************************************/
/***************** DOMESTIC HOT WATER COMPONENT ********************/

/******************** HOT WATER TANK *********************/
/**************** Water Use flow rate in and out are equal *****************/
// rho - density of water
// Cp - specific heat capacity
// vFR - volume Flow Rate
// Tin - temperature (K) of water coming in
// Tout - temperature (K) of water in tank/vessel
//static float waterUseHeatTransfer(float rho, float Cp, float vFR, float Tin, float Tout)
//{
//    float q_wt = rho*Cp*vFR*(Tin-Tout); // kg/m3 . J/K.kg . m3/s . K  (units cancel to J/s or W)
//    return q_wt;
//}
/************************************************************/

/**************** Water lost to drains *****************/
// rho - density of water
// Cp - specific heat capacity
// vFR - volume Flow Rate
// Tout - temperature (K) of water in vessel lost to drains
static float waterUseEnergyLossToDrains(float rho, float Cp, float vFRo, float Tout, float timeResolution)
{
    float q_wt = rho*Cp*Tout*(vFRo*timeResolution); // kg/m3 . J/K.kg . K . (m3/s . s) (units cancel to J)
    return q_wt;
}
/************************************************************/

/**************** END DOMESTIC HOT WATER COMPONENT *****************/
/*******************************************************************/

/************************************************************/
// Indoor Convection on wall surfaces: convective heat trasfer between air node and wall node(s)
// h - convection coefficient for wall surface type (i.e. solid wall, window)
// A - total surface area of wall surface type
// Twi - wall surface temperature (K) for surface
// Ti - internal air temperature (K)

static float indoorConvectionHeatTransfer(float h, float A, float Twi, float Ti)
{
    return h*A*(Ti-Twi);
}

/************************************************************/

/************************************************************/
// Indoor convection on surfaces of objects and internal walls in building. These are considered together as a single mass object with equal temperature.
// h - convection coefficient applied for all internal objects
// A - the total internal surface area of internal objects
// Tio - surface temperature (K) of internal objects
// Ti - indoor air temperature (K)
//
// return - heat transfer to internal objects

static float internalConvectionHeatTransfer(float h,float A,float Tio,float Ti)
{
    return (h*A*(Ti - Tio));
}
/************************************************************/

/************************************************************/
// Indoor radiative exchange between wall surfaces and mass internal object
// NOTE: This is not yet implemented!

static float indoorRadiativeHeatTransfer()
{
    float q=0.0;
    return q;
}
/************************************************************/

/************************************************************/
// Outdoor Convection on surfaces: convective heat transfer between outside wall surface and ambient air node
// h -  convection coefficient for  outside wall surface type (i.e. solid wall, window)
// A - total surface area of  outside wall surface type
// Two - wall outdoor-surface temperature (K)
// Ta - ambient air temperature (K) (should be determined outside this model)

static float outdoorConvectionHeatTransfer(float h,float A,float Two,float Ta)
{
    return h*A*(Two - Ta);
}

/************************************************************/

/************************************************************/
// Outdoor Radiation from outdoor surface
// f - list of view factors for wall
// A - building wall surface area for each wall component
// emis - emissivity of surface NOTE: Considered constant for all surfaces here, not implemented as list
// Two - outdoor surface temperature (K)s for each wall component - must be in kelvin
// Ts - list of surface temperature (K)s for each surface with which the wall component has a view (i.e. coressponding list to f) - must be in Kelvin

static float outdoorRadiativeHeatTransfer(float f, float A, float emis, float Two, float Ts){
    float q = 0.0;
    
    q = A*f*sigma*emis*(pow(Two,4) - pow(Ts,4));
// for i in range(f):
// q += A*f[i]*self.sigma*emis*(Two**4 - Ts[i]**4)
    return q;
}
/************************************************************/

/************************************************************/
// Window Solar Insolation
// Irr - Irradiance incident on window
// Tr - Effective Transmissivity
// NOTE: This should be added as heat gain to internal single mass object

static float windowInsolation(float Irr, float Tr)
{
    return Irr*Tr;
}
/************************************************************/

/************************************************************/
//  Solar Insolation on surface
// Irr - Irradiance incident on window
// Ab - Effective Absorptance of surface
// NOTE: This should be added as heat gain to external surface of wall

static float wallInsolation(float Irr, float Ab)
{
 return Irr*Ab;
}
/************************************************************/

/************************************************************/
// Wall Component Conduction (can be floor/roof/wall/window)
// k_eff - conductivity for lumped wall
// A -  total surface area of wall
// Twi - indoor wall surface temperature (K)
// Two - outdoor wall surface temperature (K)
// L - wall thickness

static float wallConduction(float k_eff, float A, float Twi, float Two, float L)
{
    return k_eff*A*((Twi-Two)/L);    // Assuming 1-d conduction in  wall component
}
/************************************************************/

/************************************************************/
// Window Component Conduction
// k_eff - list of thermal conductivity for each window surface type
// A - list of total surface area of each window type
// Twi - list of indoor window surface temperature (K) for each window type
// Two - list of outdoor window surface temperature (K) for each window type
// L - window unit thickness
// Calls wallConduction() function

static float windowConduction(float k_eff, float A, float Twi, float Two, float L)
{
    return wallConduction(k_eff,A,Twi,Two,L);
}
/************************************************************/

/************************************************************/
// Heating injected to building
// Ts - set point temperature (K) for heating load
// Ti - indoor air node temperature (K)
// P - maximum power rating of the total heating system
// epsilon - rated efficiency of the total heating system

static float heating(float Ts, float Ti, float epsilon, float P)
{
    float q_heating = 0;
    if (Ti < Ts){
        q_heating = (P - (P/exp((Ts-Ti))))*epsilon; // Designed to ramp power exponentially according to temperature (K) difference.
    }
//    printf("heatingSetPointIssue: %f  Ts: %f  Ti: %f  \n", q_heating, Ts, Ti);
    return q_heating;
}
/************************************************************/

/************************************************************/
// Building Ventilation rate heat transfer (i.e. not recirculated air)
// V - volumetric flow rate m3/s
// rho - air density
// Cp - specific heat capacity of air at constant pressure
// To - outdoor temperature (K)
// Ti - indoor temperature (K)
// return the heat flux resulting in the building (that outside the building is just the inverse sign of this value - energy balance)

static float ventilationHeatTransfer(float rho, float Cp, float V, float To, float Ti)
{
    float q_in = rho*Cp*V*(To-Ti); // kg/m3 . J/K.kg . m3/s . K  (units cancel to J/s or W)
    return q_in;
}
/************************************************************/

/************************************************************/
// Calculates the additional heat from the heating system due to system efficiency
// q_heating - heating applied to building
// epsilon - rated efficiency of the total heating system

static float additionalSystemHeatingEnergy(float q_heating, float epsilon)
{
    float qH_additional = 0;
    if (epsilon <= 1.0){
        qH_additional = (q_heating/epsilon) - q_heating;
    } else{
        qH_additional = q_heating/epsilon;
    }

    return qH_additional;  // return this value and determine is ejected immmediately outside or is released inside building
}
/************************************************************/

/************************************************************/
// Cooling of building (heat ejected)
// Ts - set point temperature (K) for cooling load
// Ti - indoor air node temperature (K)
// COP - rated efficiency of the total heating system
// P - maximum power rating of the total heating system

static float cooling(float Ts, float Ti, float COP, float P)
{
    float q_cooling = 0.0;
    if (Ti > Ts){
    q_cooling = P - (P/exp((Ti-Ts))); // Designed to ramp power exponentially according to temperature difference.
    }
    return q_cooling;
}
/************************************************************/

/************************************************************/
// Calculates the additional heat from the cooling system due to system COP
// q_cooling - cooling applied to building
// COP - rated efficiency of the total heating system

static float additionalSystemCoolingEnergy(float q_cooling, float COP){
    return (q_cooling/COP); // return this value and determine if ejected immmediately outside or is released inside building
}
/************************************************************/

/************************************************************/
// Calculates the internal gains from building occupants
// Occupants - number of occupants in building
// metRate - the metabolic rate of building occupants
// LSR - the latent heat to Sensible Heat Ratio (LH/SH)
//
// returns the latent heat and sensible heat from all occupants

static void internalOccupancyGains(int Occupants, float metRate, float LSR, float *qSen, float *qLat)
{
    *qSen = (metRate*Occupants)/(1+LSR);
    *qLat = (metRate*Occupants)*LSR/(1+LSR);
    
//    return [(metRate*Occupants)/(1+LSR), (metRate*Occupants)*LSR/(1+LSR)]; // Qtotal = metRate*Occupants, qSensible = (Qtotal)/(1+LH/SH), qLatent=qSensible*(LH/SH)
}
/************************************************************/

/************************************************************/
// Calculates the internal gains for total n appliances of given Power, P, with usage factor, f
// P - power rating of appliances
// f - usage factor of appliance
// n - number of appliances
//
// return total energy of appliances - assume all goes to heat (sensible)

static float internalApplianceGains(float P, float f, int n)
{
    return P*f*n;
}
/************************************************************/

/************************************************************/
//Run Timestep calculation for calculating heat fluxes
static PyObject *timeStepCalculation(PyObject *self, PyObject *args)
{
    float Ta, Tg, Tsurf, rho_OutsideAir, Cp_OutsideAir, Irr;
    /*** DOMESTIC HOT WATER ***/
    float Twt; // Water temperature (K) in Hot Water Tank
    float Twt_wallIn; // Hot water tank internal wall temperature (K)
    float Twt_wallExt; //Hot water tank external wall temperature (K)
    float dTwt = 0.0, dTwt_wallIn = 0.0, dTwt_wallExt = 0.0;
    float hwtThickness; //Hot water tank wall thickness
    
    float Twt_in; // Water temperature (K) of Water coming into the Water Tank
    float Vwt; // Volume of Water in Hot Water Tank (m^3)
    float Awt; // Surface Area of Hot Water Tank(m^2)
    float wVwt; // Wall volume of Hot Water Tank(m^2)
    float wtTs; // Water Tank setpoint temperature (K)
    
    float Tdhw; // Water temperature (K) of water held in use in Building
    float Tdhw_wallIn; // Hot water tank internal wall temperature (K)
    float Tdhw_wallExt; //Hot water tank external wall temperature (K)
    float dTdhw = 0.0, dTdhw_wallIn = 0.0, dTdhw_wallExt = 0.0;
    float dhwThickness; //DHW vessels wall thickness
    
    float Vdhw; // Volume of water held in use in building
    float dVdhw = 0.0; // Change in volume of Domestic Hot Water held in use in building
    float Adhw; // Surface Area of Hot Water in Vessels in Building
    float wVdhw; // Wall volume of Hot water Vessels in Building
    float VwtFlowRate; // Hot Water Flow Rate in m^3 / s
    float VdhwFlowRate; // Draining of Domestic Hot Water held in building
    
    float DHW_SHC; //Specific Heat Capacity of Domestic Hot Water
    float HWT_SHC; //Specific Heat Capacity of Hot Water Tank wall
    float VHW_SHC; //Specific Heat Capacity of Vessels containing DHW in use in Building
    
    float DHW_Density; //Density of water
    float HWT_Density; //Density of hot water tank wall
    float VHW_Density; //Density of vessels containing DHW in use in buildings
    
    float WTBVF; //water tank - building wall view factor
    float WTMVF; //water tank - building internal mass view factor
    
    float hotWaterTankWallConductivity; // Effective Wall conductivity of the Hot Water Tank
    float hotWaterTankWallConvectionCoefficient_Internal; // Effective Internal Wall convection coefficient of the Hot Water Tank
    float hotWaterTankWallConvectionCoefficient_External; // Effective External Wall convection coefficient of the Hot Water Tank
    float hotWaterTankWallEmissivity_External; // Effective External Wall emissivity of the Hot Water Tank
    float domesticHotWaterConductivity; // Effective Conductivity of vessels containing DHW in use in Building.
    float domesticHotWaterVesselsConvectionCoefficient_Internal; // Effective Internal Wall convection coefficient of the Vessels holding DHW in use in Building
    float domesticHotWaterVesselsConvectionCoefficient_External; // Effective Enternal Wall convection coefficient of the Vessels holding DHW in use in Building
    float domesticHotWaterVesselsEmissivity_External; // Effective External Wall emissivity of hot water being used within building
    /** NOTE THAT LATENT HEAT FLUX RELATING TO HOT WATER USE CURRENTLY NOT IMPLEMENTED **/
    
    float maxHeatingPower_DHW;
    float DHW_HeatingEfficiency;
    /*** END DOMESTIC HOT WATER ***/
    
    float winT; // window transmisivity
    float winA; // window absorptivity
    float winR; // window reflectivity
    float walT; // wall transmisivity
    float walA; // wall absorptivity
    float walR; // wall reflectivity
    
    float Qf_sens[2]; //List sensible Qf = [anthropogenic heatflux to air, anthropogenic heatflux to ground]
    float Qf_lat; //Latent Qf
    float Qheat; // currently only sensible but this needs to be  split into sensible and latent heat components
    float Qcool; // currently only sensible but this needs to be  split into sensible and latent heat components
    float ApplianceLoad; //sensible
    float OccupantLoad[2]; //sensible and latent heat, respectively

//    float sigma = 5.670E-8; //Stefan-Boltzmann constant (W/m^2.K^4)
//
//
    float height, width, depth, WWR, wallThickness, floorThickness, groundDepth, windowThickness;
    float wallRoofInternalConvectionCoefficient, internalMassConvectionCoefficient, floorInternalConvectionCoefficient, windowInternalConvectionCoefficient, wallRoofExternalConvectionCoefficient, windowExternalConvectionCoefficient;
    float wallRoofEffectiveConductivity, groundFloorEffectiveConductivity, windowEffectiveConductivity, externalGroundConductivity;
    float wallRoofDensity, groundFloorDensity, windowDensity, internalMassDensity, indoorAirDensity;
    float wallRoofSHC, groundFloorSHC, windowSHC, internalMassSHC, indoorAirSHC;
    float wallRoofExternalEmissivity, wallRoofInternalEmissivity, internalMassEmissivity, windowExternalEmissivity, windowInternalEmissivity;
    float windowTransmissivity, windowAbsorbtivity, windowReflectivity;
    float wallTransmisivity, wallAbsorbtivity, wallReflectivity;
    float outdoorWallBVF, outdoorWallGVF, outdoorWallSVF;
    int Occupants;
    float MetRate;
    float LSR;
    float ApplianceRating;
    int totalAppliances;
    float applianceUsageFactor;
    float maxHeatingPower, heatingEfficiency;
    float maxCoolingPower, coolingCOP;
    float airVolume;
    float ventilationRate; //Fixed at begining to have no natural ventilation. Given in units of volume of air per hour
    float wallArea, wallVolume;
    float floorArea, floorVolume;
    float windowArea, windowVolume;
    float internalMassVolume, internalMassArea; //Assumed internal mass as a cube
//
    float Ti , Tm, Twi, Two, Twii, Twio, Tfi, Tfo;
    float dTi = 0.0, dTm = 0.0, dTwi = 0.0, dTwo = 0.0, dTwii = 0.0, dTwio = 0.0, dTfi = 0.0, dTfo = 0.0;
    
    /*** DOMESTIC HOT WATER ***/
    float qhwi = 0.0; // heat flux to internal wall of hot water tank
    float qhwo_c = 0.0; // convective heat flux to external wall of hot water tank
    float qhwo_rw = 0.0, qhwo_rm = 0.0; // radiative heat flux to external wall of hot water tank
    float qhwtCon = 0.0; // heat flux through wall of hot water tank
    float qhwtTot; // total heat input into water of hot water tank over simulation, hence do not equate to zero
    float qdhwi = 0.0; // heat flux to internal wall of vessels holding DHW in use in building
    float qdhwCon = 0.0; // heat flux through wall of vessels holding DHW in use in building
    float qdhwo_c = 0.0; // convective heat flux to external wall of vessels holding DHW in use in building
    float qdhwo_rw = 0.0, qdhwo_rm = 0.0; // radiative heat flux to external wall of vessels holding DHW in use in building
    float qhwt_d = 0.0; //Heat loss as water held in use in building drains to sewer
    float qhwt_a = 0.0; // additional heat release from efficieny losses/gains of heating hot water
    
    float Q_hwt = 0.0, Q_hwtWi = 0.0, Q_hwtWo = 0.0, Q_dhw = 0.0, Q_dhwWi = 0.0, Q_dhwWo = 0.0;
    float qhwt_timestep=0.0;
    
    float VARatio_DHWVessel = 0.0;
    float minVdhw;
    /************************************************************/
    /*** END DOMESTIC HOT WATER ***/
    
    float Ts[2] = {}; //Heating and Cooling setpoint temperature (K)s, respectively
    
    int timestep, resolution;
    
    float Qf_sensible_timestep = 0.0;
    float Qf_latent_timestep = 0.0;
    float Qf_ground_timestep = 0.0;
    float q_heating_timestep = 0.0;
    float q_cooling_timestep = 0.0;
    
    float qwii = 0.0, qwia = 0.0, qwi = 0.0, qmc = 0.0, qir = 0.0, qirw = 0.0, qirf = 0.0;
    float qia = 0.0, qvr = 0.0, qwc = 0.0, qwic = 0.0, qfc = 0.0;
    float qha = 0.0, qwcon = 0.0, qwicon = 0.0, qfcon = 0.0, qgcon = 0.0;
    float qwor = 0.0, qwior = 0.0, qwoc = 0.0, qwioc = 0.0;
    
    //STS - 31/07/2018: Added parameters to return surface fluxes for each timestep.
    //This allows model user to calculate QfB and Qs externally to the model
    //and provide a more flexible interface to surface energy balance models.
    float qwii_tstepTotal = 0.0, qwia_tstepTotal = 0.0, qwi_tstepTotal = 0.0, qmc_tstepTotal = 0.0, qir_tstepTotal = 0.0, qirw_tstepTotal = 0.0, qirf_tstepTotal = 0.0;
    float qia_tstepTotal = 0.0, qvr_tstepTotal = 0.0, qwc_tstepTotal = 0.0, qwic_tstepTotal = 0.0, qfc_tstepTotal = 0.0;
    float qha_tstepTotal = 0.0, qwcon_tstepTotal = 0.0, qwicon_tstepTotal = 0.0, qfcon_tstepTotal = 0.0, qgcon_tstepTotal = 0.0;
    float qwor_tstepTotal = 0.0, qwior_tstepTotal = 0.0, qwoc_tstepTotal = 0.0, qwioc_tstepTotal = 0.0;
    float q_cooling_timestepTotal=0.0, qsensible_timestepTotal =0.0, qlatent_timestepTotal = 0.0;
    
    float qsensible = 0.0, qlatent = 0.0;
    
    float Q_mass = 0.0, Qi = 0.0, Qwi = 0.0, Qwo = 0.0;
    float Qwindowi = 0.0, Qwindowo = 0.0, Qfloori = 0.0, Qflooro = 0.0;
    
    /****************************************************/
    //Assign arguments passed in to function from Python
    if (!PyArg_ParseTuple(args, "ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffiffffffffffffffffffffffffffffiiiffffffffffffffffffffffffffffffffffffff", &Ta, &Tg, &Tsurf, &rho_OutsideAir, &Cp_OutsideAir, &Irr, &winT, &winA, &winR, &walT, &walA, &walR, &Qf_sens[0], &Qf_sens[1], &Qf_lat, &Qheat, &Qcool, &ApplianceLoad, &OccupantLoad[0], &OccupantLoad[1], &height, &width, &depth, &WWR, &wallThickness, &floorThickness, &groundDepth, &windowThickness, &wallRoofInternalConvectionCoefficient, &internalMassConvectionCoefficient, &floorInternalConvectionCoefficient, &windowInternalConvectionCoefficient, &wallRoofExternalConvectionCoefficient, &windowExternalConvectionCoefficient, &wallRoofEffectiveConductivity, &groundFloorEffectiveConductivity, &windowEffectiveConductivity, &externalGroundConductivity, &wallRoofDensity, &groundFloorDensity, &windowDensity, &internalMassDensity, &indoorAirDensity, &wallRoofSHC, &groundFloorSHC, &windowSHC, &internalMassSHC, &indoorAirSHC, &wallRoofExternalEmissivity, &wallRoofInternalEmissivity, &internalMassEmissivity, &windowExternalEmissivity, &windowInternalEmissivity, &windowTransmissivity, &windowAbsorbtivity, &windowReflectivity, &wallTransmisivity, &wallAbsorbtivity, &wallReflectivity, &outdoorWallBVF, &outdoorWallGVF, &outdoorWallSVF, &Occupants, &MetRate, &LSR, &ApplianceRating, &applianceUsageFactor, &maxHeatingPower, &heatingEfficiency, &maxCoolingPower, &coolingCOP, &airVolume, &ventilationRate, &wallArea, &wallVolume, &floorArea, &floorVolume, &windowArea, &windowVolume, &internalMassVolume, &internalMassArea, &Ti , &Tm, &Twi, &Two, &Twii, &Twio, &Tfi, &Tfo, &Ts[0], &Ts[1], &totalAppliances, &timestep, &resolution, &qhwtTot, &Twt, &Twt_wallIn, &Twt_wallExt, &hwtThickness, &Twt_in, &Vwt, &Awt, &wVwt, &wtTs, &Tdhw, &Tdhw_wallIn, &Tdhw_wallExt, &dhwThickness, &Vdhw, &Adhw, &wVdhw, &VwtFlowRate, &VdhwFlowRate, &DHW_SHC, &HWT_SHC, &VHW_SHC, &DHW_Density, &HWT_Density, &VHW_Density, &WTBVF, &WTMVF, &hotWaterTankWallConductivity, &hotWaterTankWallConvectionCoefficient_Internal, &hotWaterTankWallConvectionCoefficient_External, &hotWaterTankWallEmissivity_External, &domesticHotWaterConductivity, &domesticHotWaterVesselsConvectionCoefficient_Internal, &domesticHotWaterVesselsConvectionCoefficient_External, &domesticHotWaterVesselsEmissivity_External, &maxHeatingPower_DHW, &DHW_HeatingEfficiency, &minVdhw)) return NULL;
    /****************************************************/
    
    //Used to recalculate Area of DHW in use
    if (Adhw>0.0){
        VARatio_DHWVessel = Vdhw/Adhw;
    }
    
    if ((timestep % resolution) == 0) {
        for (int i=0;i<(timestep/resolution);i++) {
            qwii = windowInsolation(Irr, winT);
            qwia = windowInsolation(Irr, winA);
            qwi = wallInsolation(Irr, walA);
//            printf("qmc: %f  Tm: %f  Ti: %f  ", qmc, Tm, Ti);
            qmc = internalConvectionHeatTransfer(internalMassConvectionCoefficient, internalMassArea, Tm, Ti);
//            printf("qmc: %f  Tm: %f  Ti: %f \n", qmc, Tm, Ti);
            qir = indoorRadiativeHeatTransfer(); //  for wall internal radiative exchange
            qirw = qir; //  for window internal radiative exchange - TODO: currently no distinction in internal radiative exchanges
            qirf = qir; //  for ground floor internal radiative exchange - TODO: currently no distinction in internal radiative exchanges
//
            qia = internalApplianceGains(ApplianceRating, applianceUsageFactor, totalAppliances);
            qvr = ventilationHeatTransfer(indoorAirDensity,indoorAirSHC, ventilationRate, Ta, Ti);
            qwc = indoorConvectionHeatTransfer(wallRoofInternalConvectionCoefficient,wallArea,Twi,Ti);
            qwic = indoorConvectionHeatTransfer(windowInternalConvectionCoefficient,windowArea,Twii,Ti);
            qfc = indoorConvectionHeatTransfer(floorInternalConvectionCoefficient,floorArea,Tfi,Ti);
//
            q_heating_timestep = heating(Ts[0], Ti, heatingEfficiency, maxHeatingPower);
            q_cooling_timestep = cooling(Ts[1], Ti, coolingCOP, maxCoolingPower);
            
            internalOccupancyGains(Occupants, MetRate, LSR, &qsensible, &qlatent);
            
            qha = additionalSystemHeatingEnergy(q_heating_timestep, heatingEfficiency);
            qwcon = wallConduction(wallRoofEffectiveConductivity, wallArea, Twi, Two, wallThickness);
            qwicon = windowConduction(windowEffectiveConductivity, windowArea, Twii, Twio, windowThickness);
            qfcon = wallConduction(groundFloorEffectiveConductivity, floorArea, Tfi, Tfo, floorThickness);
            qgcon = wallConduction(externalGroundConductivity, floorArea, Tfo, Tg, groundDepth); // conduction from ground floor to external ground - depth of the ground can be set (groundDepth) and ground temperature should be determined accordingly..
            
            qwor = outdoorRadiativeHeatTransfer(outdoorWallBVF, wallArea, wallRoofExternalEmissivity, Two, Tsurf);
            qwior = outdoorRadiativeHeatTransfer(outdoorWallBVF, windowArea, windowExternalEmissivity, Twio, Tsurf);
            
            qwoc = outdoorConvectionHeatTransfer(wallRoofExternalConvectionCoefficient, wallArea, Two, Ta);
            qwioc = outdoorConvectionHeatTransfer(windowExternalConvectionCoefficient, windowArea, Twio, Ta);
            
            /**************************/
            /*** DOMESTIC HOT WATER ***/
            if (Vwt > 0.0){
                
                // convective heat flux to internal wall of hot water tank
                qhwi = indoorConvectionHeatTransfer(hotWaterTankWallConvectionCoefficient_Internal,Awt,Twt_wallIn,Twt);
                
                // heat flux by conduction through wall of hot water tank
                qhwtCon = wallConduction(hotWaterTankWallConductivity, Awt, Twt_wallIn, Twt_wallExt, hwtThickness);
                
                // convective heat flux for external wall of hot water tank
                qhwo_c = outdoorConvectionHeatTransfer(hotWaterTankWallConvectionCoefficient_External, Awt, Twt_wallExt, Ti);
                // radiative heat flux for external wall of hot water tank
                //TODO: Should expand to consider windows and floor as well.
                qhwo_rw = outdoorRadiativeHeatTransfer(WTBVF, Awt, hotWaterTankWallEmissivity_External, Twt_wallExt, Twi); // to building walls
                qhwo_rm = outdoorRadiativeHeatTransfer(WTMVF, Awt, hotWaterTankWallEmissivity_External, Twt_wallExt, Tm); // to internal mass
                
                // heat input into water of hot water tank
                qhwt_timestep = heating(wtTs, Twt, DHW_HeatingEfficiency, maxHeatingPower_DHW);
                
                //Heat release from hot water heating due to efficiency losses
                qhwt_a = additionalSystemHeatingEnergy(qhwt_timestep, DHW_HeatingEfficiency);
            }
            if (Vdhw > 0.0){
                // heat flux to internal wall of vessels holding DHW in use in building
                qdhwi = indoorConvectionHeatTransfer(domesticHotWaterVesselsConvectionCoefficient_Internal,Adhw,Tdhw_wallIn,Tdhw);
                
                // heat flux by conduction through wall of vessels holding DHW in use in building
                qdhwCon = wallConduction(domesticHotWaterConductivity, Adhw, Tdhw_wallIn, Tdhw_wallExt, dhwThickness);
                
                // convective heat flux to external wall of vessels holding DHW in use in building
                qdhwo_c = outdoorConvectionHeatTransfer(domesticHotWaterVesselsConvectionCoefficient_External, Adhw, Tdhw_wallExt, Ti);
                // radiative heat flux to external wall of vessels holding DHW in use in building
                // TODO: Should expand to consider windows and floor as well.
                qdhwo_rw = outdoorRadiativeHeatTransfer(WTBVF, Adhw, domesticHotWaterVesselsEmissivity_External, Tdhw_wallExt, Twi);
                qdhwo_rm = outdoorRadiativeHeatTransfer(WTMVF, Adhw, domesticHotWaterVesselsEmissivity_External, Tdhw_wallExt, Tm);
                
                //Heat transfer due to use and replacement of water
                //qhwt_v = waterUseHeatTransfer(DHW_Density, DHW_SHC, VwtFlowRate, Twt_in, Twt);
                
            } else if ((Vdhw == minVdhw) && (VwtFlowRate < VdhwFlowRate)){
                //Set Drain Flow rate to be same as usage flow rate to avoid hot water storage going below the set minimum threshold (minVdhw)
                VdhwFlowRate = VwtFlowRate;
            }
            

            //TODO: Need to consider the latent component of heat transfer for the DHW in use here also.
            
            /************************************************************/
            // Need to calculate the temperature change in the hot water tank due to energy flux
            Q_hwt = qhwt_timestep - qhwi;
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change in the hot water tank internal walls to energy flux
            Q_hwtWi = qhwi - qhwtCon;
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change in the hot water tank external walls to energy flux
            Q_hwtWo = qhwtCon - qhwo_c - qhwo_rw - qhwo_rm;
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change in the DHW vessels (i.e. in use hot water) due to energy flux
            Q_dhw = - qdhwi;
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change in tthe DHW vessels (i.e. in use hot water) internal walls to energy flux
            Q_dhwWi = qdhwi - qdhwCon;
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change in the hot water tank external walls to energy flux
            Q_dhwWo = qdhwCon - qdhwo_c - qdhwo_rw - qdhwo_rm;
            /************************************************************/
            
            /*** END DOMESTIC HOT WATER ***/
            /******************************/
            
            /************************************************************/
            // Need to calculate the temperature change in the internal mass object due to energy flux, given its heat capacity
            Q_mass = qwii + qmc + qir + qhwo_rm + qdhwo_rm;
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change in the internal volume of air due to energy flux to/from internal mass object AND to/from internal walls.
            // and to/from hot water tanks and to/from the DHW held in use in the building.
            Qi = qia + qsensible + qvr + q_heating_timestep - q_cooling_timestep - qmc - qir - qwc - qwic - qfc + qha + qhwo_c + qdhwo_c + qhwt_a;
            // TODO: Have not yet considered the latent heat component from occupancy gains!!
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change of the internal wall surface due to internal heat exchanges and conduction through wall. Need an internal wall thermal mass also.
            Qwi = qwc - qwcon - qir + qhwo_rw + qdhwo_rw;
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change of the external wall surface due to the conduction through the wall as well as wall surface exchanges with outside environment. Need an external wall thermal mass also.
            Qwo = qwcon + qwi - qwor - qwoc;
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change of the internal window surface due to internal heat exchanges and conduction through wall. Need an internal window thermal mass also.
            Qwindowi = qwic - qwicon - qirw;
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change of the external window surface due to the conduction through the window as well as window surface exchanges with outside environment. Need an external window thermal mass also.
            Qwindowo = qwicon + qwia - qwior - qwioc;
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change of the internal ground floor surface due to internal heat exchanges and conduction through floor. Need an internal floor thermal mass also.
            Qfloori = qfc - qfcon - qirf;
            
            /************************************************************/
            
            /************************************************************/
            // Need to calculate the temperature change of the external ground floor surface due to the conduction through the floor as well as surface exchanges with outside ground environment. Need an external wall thermal mass also.
            Qflooro = qfcon - qgcon;
            
            /************************************************************/
            
            /************************************************************/
            // Detected Qf for SUEWS will be the outside wall and window heat flux, ventilation losses, and any additional HVAC system heat ejection related to system COP/efficiency.
            // TODO: Consider Heat Storage Flux in this calculation. Where these fluxes are negative they represent the building absorbing heat from outside. This is part of Storage heat flux. There is also a question of how (or even if you can)  separate storage flux from QfB for solar passive gains.
            if (qwor < 0) qwor=0;
            if (qwior < 0) qwior=0;
            if (qwoc < 0) qwoc=0;
            if (qwioc < 0) qwioc=0;
            if (qvr > 0) qvr=0;
            Qf_sensible_timestep = (qwor + qwior + qwoc + qwioc - qvr + q_cooling_timestep + additionalSystemCoolingEnergy(q_cooling_timestep, coolingCOP)) * resolution; //  Qf is total  heat flux from building (J), hence the multiplication by resolution (in seconds)  - removed repetition of qwor + qwior (30/07/2018)
            // TODO: term for delqS if above conditions are satisfied
            Qf_latent_timestep = qlatent; // TODO: Need to separate latent and sensible flux
            
            Qf_ground_timestep = qgcon * resolution;
            
            /************************************************************/
            // Adds timestep heat flux to overall heat flux for building model
            Qf_sens[0] += Qf_sensible_timestep; //to atmosphere
            Qf_sens[1] += Qf_ground_timestep; //to ground
            Qf_lat += Qf_latent_timestep;
            /************************************************************/
            
            /************************************************************/
            // STS - 31/0718: Adds time resolution heat flux to all building Energy Exchanges
            // for building model timestep. Allows for calculating QfB and
            // Qs outside of building energy model.
            qwii_tstepTotal += qwii*resolution;
            qwia_tstepTotal += qwia*resolution;
            qwi_tstepTotal += qwi*resolution;
            qmc_tstepTotal += qmc*resolution;
            qir_tstepTotal += qir*resolution;
            qirw_tstepTotal += qirw*resolution;
            qirf_tstepTotal += qirf*resolution;
            qia_tstepTotal += qia*resolution;
            qvr_tstepTotal += qvr*resolution;
            qwc_tstepTotal += qwc*resolution;
            qwic_tstepTotal += qwic*resolution;
            qfc_tstepTotal += qfc*resolution;
            qha_tstepTotal += qha*resolution;
            qwcon_tstepTotal += qwcon*resolution;
            qwicon_tstepTotal += qwicon*resolution;
            qfcon_tstepTotal += qfcon*resolution;
            qgcon_tstepTotal += qgcon*resolution;
            qwor_tstepTotal += qwor*resolution;
            qwior_tstepTotal += qwior*resolution;
            qwoc_tstepTotal += qwoc*resolution;
            qwioc_tstepTotal += qwioc*resolution;
            q_cooling_timestepTotal += (q_cooling_timestep + additionalSystemCoolingEnergy(q_cooling_timestep, coolingCOP))*resolution;
            qsensible_timestepTotal += qsensible*resolution;
            qlatent_timestepTotal += qlatent*resolution;
            /************************************************************/
            
            /************************************************************/
            // Adds timestep heating/cooling to overall heating/cooling for building model
            Qheat += (q_heating_timestep * resolution);
            Qcool += (q_cooling_timestep * resolution);
            
            /***** DOMESTIC HOT WATER HEATING *****/
            qhwtTot += (qhwt_timestep * resolution);
            /**************************************/
            
            /************************************************************/
            
            /*******************************************************************/
            //Building temperature changes calculated for each lumped component
            /*******************************************************************/
            /************************************************************/
            // Temperature Changes calculated for next timestep
            
            /****************************/
            /**** DOMESTIC HOT WATER ****/
            
            // temperature (K) of DHW in use in Building due to heat transfer to building
            if (Vdhw > 0.0){
                dTdhw = (Q_dhw/(DHW_Density*DHW_SHC)*Vdhw)*resolution;
                Tdhw = Tdhw + dTdhw;
            }
                
            //DHW "vessel" Internal Wall surface temperature (K)
            if (wVdhw > 0.0){
            dTdhw_wallIn = (Q_dhwWi/((VHW_Density*VHW_SHC)*(wVdhw/2)))*resolution;
            Tdhw_wallIn = Tdhw_wallIn + dTdhw_wallIn;
            
            //DHW "vessel" External Wall surface temperature (K)
            dTdhw_wallExt = (Q_dhwWo/((VHW_Density*VHW_SHC)*(wVdhw/2)))*resolution;
            Tdhw_wallExt = Tdhw_wallExt + dTdhw_wallExt;
            }

            //Heat transfer to sewer/drain based on water in building going to drain.
            qhwt_d = waterUseEnergyLossToDrains(DHW_Density, DHW_SHC, VdhwFlowRate, Tdhw, resolution);
            
            //Need to recalculate the volume of DHW in use in building before calculating the temperature (K) of DHW in use.
            dVdhw = (VwtFlowRate - VdhwFlowRate)*resolution;
            Vdhw = Vdhw + dVdhw;
            
            //Avoid going below a given minimum volume
            if (Vdhw < minVdhw){
                Vdhw = minVdhw;
                
            }
            
            //Need to recalculate the temperature after mixing of water
            Tdhw = (((VwtFlowRate*resolution)*Twt) + ((Vdhw - (VwtFlowRate*resolution))*Tdhw)) / Vdhw;
            
            /**********/
            //Recalculate DHW vessel surface area and wall volume based on new volume and maintaining the previous
            if (Vdhw > 0.0){ //Checks that Volume isn't zero
                Adhw = Vdhw/VARatio_DHWVessel;
            }
            else { // if Volume is zero it makes the area also zero
                Adhw = 0.0;
            }
            wVdhw = Adhw*dhwThickness;
            /**********/

            //Hot Water Tank water temperature (K)
            if (Vwt > 0.0) {
                dTwt = (Q_hwt/((DHW_Density*DHW_SHC)*Vwt))*resolution;//(Q_hwt/((DHW_Density*DHW_SHC)*Vwt))*resolution; // resolution in seconds
                Twt = Twt + dTwt;
//                printf("HWT Water Temperature: %f, dTwt: %f, Heating Q_hwt_timestep: %f \n", Twt, dTwt, qhwt_timestep);
            }
            
            // Hot Water Tank Internal Wall surface temperature (K)
            dTwt_wallIn = (Q_hwtWi/((HWT_Density*HWT_SHC)*(wVwt/2)))*resolution;
            Twt_wallIn = Twt_wallIn + dTwt_wallIn;
            
            // Hot Water Tank External Wall surface temperature (K)
            dTwt_wallExt = (Q_hwtWo/((HWT_Density*HWT_SHC)*(wVwt/2)))*resolution;
            Twt_wallExt = Twt_wallExt + dTwt_wallExt;
            
            //Need to recalculate the volume of water in hot water tank before calculating the temperature (K) after mixing with mains water
            //NOTE: Currently not implemented as the water tank volume is considered constant at all times.
            //dVwt = (VwtFlowRate - VwtFlowRate)*resolution;
            //Vwt = Vwt + dVwt;
            
            //Need to recalculate the temperature after mixing of mains water with hot water remaining in the tank
            Twt = (((VwtFlowRate*resolution)*Twt_in) + ((Vwt - (VwtFlowRate*resolution))*Twt)) / Vwt;
            
            /** END DOMESTIC HOT WATER **/
            /****************************/
            
            
            // Indoor thermal mass temperature (K)
            dTm = (Q_mass/((internalMassDensity*internalMassSHC)*internalMassVolume))*resolution; // resolution in seconds
            Tm = Tm + dTm;
            
            // Indoor air temperature (K)
            dTi = (Qi/((indoorAirDensity*indoorAirSHC)*airVolume))*resolution; // resolution in seconds
            Ti = Ti + dTi;
            
            // print "dTi: " + str(dTi)
            // print "Ti: " + str(self.Ti)
            // print "Qi: " + str(Qi)
            // print "q_heating_timestep: " + str(q_heating_timestep)
            
            // Internal wall surface temperature (K)
            dTwi = (Qwi/((wallRoofDensity*wallRoofSHC)*(wallVolume/2)))*resolution; // resolution in seconds
            Twi = Twi + dTwi;
            
            // print "Twi: " + str(self.Twi)
            
            // External wall surface temperature (K)
            dTwo = (Qwo/((wallRoofDensity*wallRoofSHC)*(wallVolume/2)))*resolution;  // resolution in seconds
            Two = Two + dTwo;
            
            // Internal window surface temperature (K)
            dTwii = (Qwindowi/((windowDensity*windowSHC)*(windowVolume/2)))*resolution; // resolution in seconds
            Twii = Twii + dTwii;
            
            // External window surface temperture
            dTwio = (Qwindowo/((windowDensity*windowSHC)*(windowVolume/2)))*resolution; // resolution in seconds
            Twio = Twio + dTwio;
            
            // Internal ground floor surface temperature (K)
            dTfi = (Qfloori/((groundFloorDensity*groundFloorSHC)*(floorVolume/2)))*resolution; // resolution in seconds
            Tfi = Tfi + dTfi;
            
            // External ground floor surface temperature (K)
            dTfo = (Qflooro/((groundFloorDensity*groundFloorSHC)*(floorVolume/2)))*resolution;  // resolution in seconds
            Tfo = Tfo + dTfo;
            /************************************************************/

        }
    } else{
        printf("Timestep: %i not equally divisible by given resolution: %i.\n", timestep, resolution);
    }
    //  printf("Qheat: %f and q_heating_timestep_last: %f\n", Qheat, q_heating_timestep);
     /*******************************************************************/
     //Return Qf and component temperatures at end of timestep
     /*******************************************************************/
    return Py_BuildValue("ffffffffffffffffffffffffffffffffffffffffffffffff", Ti , Tm, Twi, Two, Twii, Twio, Tfi, Tfo, Qf_sens[0], Qf_sens[1], Qf_lat, Qheat, Qcool, qwii_tstepTotal, qwia_tstepTotal, qwi_tstepTotal, qmc_tstepTotal, qir_tstepTotal, qirw_tstepTotal, qirf_tstepTotal, qia_tstepTotal, qvr_tstepTotal, qwc_tstepTotal, qwic_tstepTotal, qfc_tstepTotal, qha_tstepTotal, qwcon_tstepTotal, qwicon_tstepTotal, qfcon_tstepTotal, qgcon_tstepTotal, qwor_tstepTotal, qwior_tstepTotal, qwoc_tstepTotal, qwioc_tstepTotal, q_cooling_timestepTotal, qhwtTot, qhwt_d, Twt, Twt_wallIn, Twt_wallExt, Tdhw, Tdhw_wallIn, Tdhw_wallExt, Vdhw, Adhw, wVdhw, qsensible_timestepTotal, qlatent_timestepTotal);
}

